@@ -26,6 +26,9 @@
  * Then the url of the webservice will be the url of your FusionDirectory instance followed by /jsonrpc.php
  */
 
+ini_set('session.use_cookies', 0);
+ini_set('session.use_only_cookies', 1);
+
 function authenticateHeader($message = 'Authentication required')
 {
   header('WWW-Authenticate: Basic realm="FusionDirectory"');
@@ -69,6 +72,7 @@ function initiateRPCSession($id = NULL, $ldap = NULL, $user = NULL, $pwd = NULL)
     $config = session::global_get('config');
     $plist  = session::global_get('plist');
     $ui     = session::global_get('ui');
+    initLanguage();
   } else {
     $config = new config(CONFIG_DIR."/".CONFIG_FILE, $BASE_DIR);
     if ($ldap === NULL) {
@@ -149,8 +153,6 @@ class fdRPCService
     }
     $this->ldap->cd($config->current['BASE']);
 
-    logging::log('debug', 'JSON-RPC', 'Method '.$method, array(), 'Params:'.print_r($params, TRUE));
-
     return call_user_func_array(array($this, '_'.$method), $params);
   }
 
@@ -164,7 +166,7 @@ class fdRPCService
     } else {
       $self = '';
     }
-    if (!$plist->check_access($infos['aclCategory'].$self)) {
+    if (!$plist->check_access(array('ACL' => $infos['aclCategory'].$self))) {
       throw new FusionDirectoryException("Unsufficient rights for accessing type '$type$self'");
     }
     if ($tabs !== NULL) {
@@ -173,20 +175,35 @@ class fdRPCService
       }
       foreach ($tabs as $tab) {
         $pInfos = pluglist::pluginInfos($tab);
-        if (!$plist->check_access(join($self.',', $pInfos['plCategory']).$self)) {
+        if ($pInfos === NULL) {
+          throw new FusionDirectoryException("Tab '$tab' of type '$type' does not exists");
+        }
+        if (!isset($pInfos['plCategory'])) {
+          throw new FusionDirectoryException("Tab '$tab' of type '$type' has no ACL category");
+        }
+        if (!$plist->check_access(array('ACL' => join($self.',', $pInfos['plCategory']).$self))) {
           throw new FusionDirectoryException("Unsufficient rights for accessing tab '$tab' of type '$type$self'");
         }
       }
     }
   }
 
+  /*!
+   * \brief Log out
+   */
+  protected function _logout ()
+  {
+    session::destroy();
+    return TRUE;
+  }
+
   /*!
    * \brief Get list of object of objectType $type in $ou
    */
   protected function _ls ($type, $attrs = NULL, $ou = NULL, $filter = '')
   {
     $this->checkAccess($type);
-    return objects::ls($type, $attrs, $ou, $filter);
+    return objects::ls($type, $attrs, $ou, $filter, TRUE);
   }
 
   /*!
@@ -194,8 +211,15 @@ class fdRPCService
    */
   protected function _count ($type, $ou = NULL, $filter = '')
   {
+    global $ui;
     $this->checkAccess($type);
-    return objects::count($type, $ou, $filter);
+    $infos  = objects::infos($type);
+    $acl    = $infos['aclCategory'].'/'.$infos['mainTab'];
+    if (strpos($ui->get_permissions($ou, $acl), 'r') !== FALSE) {
+      return objects::count($type, $ou, $filter);
+    } else {
+      return count(objects::ls($type, NULL, $ou, $filter, TRUE));
+    }
   }
 
   /*!
@@ -218,7 +242,6 @@ class fdRPCService
    */
   protected function _listTypes()
   {
-    global $config;
     $types  = objects::types();
 
     $result = array();
@@ -265,15 +288,21 @@ class fdRPCService
     } elseif (!$tabobject->by_object[$tab]->displayHeader) {
       return array('errors' => array('Tab '.$tab.' cannot be deactivated, can’t remove it'));
     } elseif (!$tabobject->by_object[$tab]->is_account) {
-      return array('errors' => array('Tab '.$tab.' is not activated on '.$dn.', can’t remove it'));
+      return array('errors' => array('Tab '.$tab.' is not activated on "'.$dn.'", can’t remove it'));
+    } elseif (!$tabobject->by_object[$tab]->acl_is_removeable()) {
+      return array('errors' => array('You don\'t have sufficient rights to disable tab "'.$tab.'"'));
+    } else {
+      list($disabled, $buttonText, $text) = $tabobject->by_object[$tab]->getDisplayHeaderInfos();
+      if ($disabled) {
+        return array('errors' => array($text));
+      }
     }
     $_POST = array($tab.'_modify_state' => 1);
     $tabobject->save_object();
-    $errors = $tabobject->check();
+    $errors = $tabobject->save();
     if (!empty($errors)) {
       return array('errors' => $errors);
     }
-    $tabobject->save();
     return $tabobject->dn;
   }
 
@@ -298,7 +327,7 @@ class fdRPCService
       $fields = $object->attributesInfo;
       foreach ($fields as &$section) {
         $attributes = array();
-        foreach ($section['attrs'] as $key => $attr) {
+        foreach ($section['attrs'] as $attr) {
           if ($object->acl_is_readable($attr->getAcl())) {
             $attr->serializeAttribute($attributes, TRUE);
           }
@@ -353,11 +382,10 @@ class fdRPCService
       $_POST[$tab.'_modify_state'] = 1;
     }
     $tabobject->save_object();
-    $errors = $tabobject->check();
+    $errors = $tabobject->save();
     if (!empty($errors)) {
       return array('errors' => $errors);
     }
-    $tabobject->save();
     return $tabobject->dn;
   }
 
@@ -382,7 +410,7 @@ class fdRPCService
       $fields = $object->attributesInfo;
       foreach ($fields as &$section) {
         $attributes = array();
-        foreach ($section['attrs'] as $key => $attr) {
+        foreach ($section['attrs'] as $attr) {
           if ($object->acl_is_readable($attr->getAcl())) {
             $attr->serializeAttribute($attributes, FALSE);
           }
@@ -423,6 +451,10 @@ class fdRPCService
           $tabobject->by_object[$tab]->displayHeader &&
           !$tabobject->by_object[$tab]->is_account
         ) {
+        list($disabled, $buttonText, $text) = $tabobject->by_object[$tab]->getDisplayHeaderInfos();
+        if ($disabled) {
+          return array('errors' => array($text));
+        }
         if ($tabobject->by_object[$tab]->acl_is_createable()) {
           $tabobject->by_object[$tab]->is_account = TRUE;
         } else {
@@ -436,11 +468,10 @@ class fdRPCService
       $tabobject->current = $tab;
       $tabobject->save_object(); /* Should not do much as POST is empty, but in some cases is needed */
     }
-    $errors = $tabobject->check();
+    $errors = $tabobject->save();
     if (!empty($errors)) {
       return array('errors' => $errors);
     }
-    $tabobject->save();
     return $tabobject->dn;
   }
 
@@ -468,11 +499,10 @@ class fdRPCService
       return array('errors' => array($error));
     }
     $tabobject = $template->apply();
-    $errors = $tabobject->check();
+    $errors = $tabobject->save();
     if (!empty($errors)) {
       return array('errors' => $errors);
     }
-    $tabobject->save();
     return $tabobject->dn;
   }
 
@@ -483,7 +513,6 @@ class fdRPCService
   {
     global $ui;
     $infos = objects::infos($type);
-    $plist = session::global_get('plist');
     // Check permissions, are we allowed to remove this object?
     $acl = $ui->get_permissions($dn, $infos['aclCategory'].'/'.$infos['mainTab']);
     if (preg_match('/d/', $acl)) {
@@ -518,7 +547,7 @@ class fdRPCService
     }
 
     foreach ($dns as $dn) {
-      if (!preg_match('/w/', $ui->get_permissions($dn, 'user/password'))) {
+      if (!preg_match('/w/', $ui->get_permissions($dn, 'user/user', 'userLock'))) {
         $disallowed[] = $dn;
       }
     }
@@ -587,7 +616,7 @@ class fdRPCService
     }
 
     foreach ($dns as $dn) {
-      if (!preg_match('/r/', $ui->get_permissions($dn, 'user/password'))) {
+      if (!preg_match('/r/', $ui->get_permissions($dn, 'user/user', 'userLock'))) {
         $disallowed[] = $dn;
       }
     }
@@ -637,7 +666,7 @@ class fdRPCService
     $pwRecovery->email_address = $email;
     $dn = $pwRecovery->step2();
     if ($pwRecovery->step == 2) { /* No errors */
-      if (!preg_match('/w/', $ui->get_permissions($dn, 'user/password'))) {
+      if (!preg_match('/w/', $ui->get_permissions($dn, 'user/user', 'userPassword'))) {
         return array('errors' => array(msgPool::permModify($dn)));
       }
       $token = $pwRecovery->generateAndStoreToken();
